package com.dx.easychat.easychatting.controller;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dx.easychat.easychatting.entity.FileEntity;
import com.dx.easychat.easychatting.service.FileEntityService;
import jakarta.annotation.Resource;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Date;
import java.util.UUID;

@RestController
@Log4j2
@RequestMapping("/api/file")
public class FileController {

    // TODO 文件上传同名问题
    // TODO 删除消息时 同步删除物理文件
    @Value("${spring.customer.config.file-path:/opt/file_temp/}")
    private String filePath;

    @Resource
    private FileEntityService fileEntityService;

    @PostMapping("/upload")
    @ResponseBody
    public String uploadImageFile(@RequestParam("file") MultipartFile multipartFile) throws IOException {
        JSONObject jsonObject = new JSONObject();
        //获取文件名
        String fileName = multipartFile.getOriginalFilename();
        //获取文件后缀名
        assert fileName != null;
        String contentType = multipartFile.getContentType();
        String fileType = "";
        int index = fileName.lastIndexOf(".");
        if (index != -1) {
            fileType = fileName.substring(index + 1);
        }
        FileEntity fileEntity = new FileEntity();
        String uuid = UUID.randomUUID().toString();
        fileEntity.setUuid(uuid);
        fileEntity.setFileName(fileName);
        String realFilePath = filePath + uuid;
        fileEntity.setFilePath(realFilePath);
        fileEntity.setUploadTime(new Date());
        fileEntity.setFileMimeType(contentType);
        fileEntity.setFileType(fileType);
        log.info("文件名：{}", fileName);
        log.info("filePath：{}", filePath);
        File uploadFile = new File(realFilePath);
        //将临时文件转存到指定磁盘位置
        multipartFile.transferTo(uploadFile);
        fileEntityService.save(fileEntity);
        jsonObject.put("code", 200);
        jsonObject.put("uuid", uuid);
        jsonObject.put("fileType", fileType);
        jsonObject.put("fileMimeType", contentType);
        return jsonObject.toJSONString();
    }

    /**
     * 下载文件:将输入流中的数据循环写入到响应输出流中，而不是一次性读取到内存
     *
     * @param fileId 文档id
     * @param response 返回体
     */
    @GetMapping("/{fileId}")
    public void downLoadFile(HttpServletResponse response, @PathVariable String fileId) throws IOException {
        response.setContentType("text/json;charset=utf-8");
        PrintWriter writer = response.getWriter();
        try {
            // 读到流中
            LambdaQueryWrapper<FileEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(FileEntity::getUuid, fileId);
            FileEntity fileEntity = fileEntityService.getOne(lambdaQueryWrapper);
            log.info("下载时的查询：{}", fileEntity);
            InputStream inputStream = Files.newInputStream(Paths.get(fileEntity.getFilePath()));
            String fileName = fileEntity.getFileName();
            response.reset();
            response.setContentType("application/octet-stream");
            response.addHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, StandardCharsets.UTF_8));
            ServletOutputStream outputStream = response.getOutputStream();
            byte[] b = new byte[1024];
            int len;
            //从输入流中读取一定数量的字节，并将其存储在缓冲区字节数组中，读到末尾返回-1
            while ((len = inputStream.read(b)) > 0) {
                outputStream.write(b, 0, len);
            }
            inputStream.close();
        } catch (Exception e) {
            log.error("下载文件异常：{}", e.getMessage());
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("code",500);
            jsonObject.put("message",e.getMessage());
            writer.write(jsonObject.toJSONString());
        }
    }
}
