package com.dx.easychat.easychatting.filter;

import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpFilter;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

@Component
@Order(-102)
@Log4j2
public class CorsCustomerFilter extends HttpFilter {

    @Value("${spring.security.cors.origin:http://localhost:5173,http://127.0.0.1:5173}")
    private String allowOrigin;

    @Override
    protected void doFilter(HttpServletRequest request,
                            HttpServletResponse response,
                            FilterChain chain) throws IOException, ServletException {
        response.addHeader("Access-Control-Allow-Origin",this.resolveOrigin(request));
        response.addHeader("Access-Control-Allow-Methods","GET, POST, PUT, DELETE, OPTIONS");
        // response.addHeader("Access-Control-Allow-Headers","Authorization, Content-Type");
        response.addHeader("Access-Control-Allow-Headers","*");
        chain.doFilter(request, response);
    }

    private void addCorsHeader(HttpServletRequest request,
                               HttpServletResponse response) {
        response.addHeader("Access-Control-Allow-Origin",allowOrigin);
        response.addHeader("Access-Control-Allow-Methods","GET, POST, PUT, DELETE, OPTIONS");
        response.addHeader("Access-Control-Allow-Headers","Authorization, Content-Type");
    }

    /**
     * 解析配置文件中的请求原始站点
     * @param request 请求
     * @return 解析得到的请求头值
     */
    private String resolveOrigin(HttpServletRequest request){
        List<String> list = Arrays.asList(allowOrigin.split(","));
        String origin = request.getHeader("Origin");
        if (list.isEmpty()) return "http://localhost:5173";
        if ("*".equals(allowOrigin) || list.contains(origin)) return origin;
        else return list.getFirst();
    }
}
